/*
	Copyright (C) 2011 Steve Thomas <steve AT tobtu DOT com>

	This file is part of MySQL323 Collider.

	MySQL323 Collider is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	MySQL323 Collider is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with MySQL323 Collider.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef COMMON_H
#define COMMON_H

#include <stdio.h>
#include "architecture.h"

#define MAX_LEN	24

#if defined(_WIN32) && !defined(__GNUC__)
	#pragma warning(disable:4996)
#endif

#define __STDC_CONSTANT_MACROS
#define __STDC_FORMAT_MACROS
#define __STDC_LIMIT_MACROS
#ifdef _WIN32
	#include "inttypes.h"
	#include <time.h>
	#include <windows.h>
	//#include <conio.h>
	#define TIMER_TYPE                clock_t
	#define TIMER_FUNC(t)             t = clock()
	#define TIMER_DIFF(s,e)           ((e - s) / (double)CLOCKS_PER_SEC)
	#define DIRECTORY_SEPERATOR       "\\"
	#define DIRECTORY_SEPERATOR_CHAR  '\\'
#else
	#include <inttypes.h>
	#include <sys/time.h>
	#include <unistd.h>
	//#include <fcntl.h>
	#define TIMER_TYPE                timeval
	#define TIMER_FUNC(t)             gettimeofday(&t, NULL)
	#define TIMER_DIFF(s,e)           ((e.tv_sec - s.tv_sec) + (e.tv_usec - s.tv_usec) / (double)1000000.0)
	#define DIRECTORY_SEPERATOR       "/"
	#define DIRECTORY_SEPERATOR_CHAR  '/'

	#ifndef PRIu64
		#define PRIu64 "llu"
	#endif
	#ifndef PRIx64
		#define PRIx64 "llx"
	#endif
	#ifndef UINT64_MAX
		#define UINT64_MAX 0xffffffffffffffffull
	#endif
	#ifndef UINT32_MAX
		#define UINT32_MAX 0xffffffff
	#endif
#endif

#ifdef ARC_x86
	#if defined(_WIN32) && !defined(__GNUC__)
		#define MUL_32_RET_64(a32,b32,r64) \
			{ \
				__asm mov eax,a32 \
				__asm mul b32 \
				__asm mov [DWORD PTR r64],eax \
				__asm mov [DWORD PTR r64+4],edx \
			}

		#define DIV_MOD_64_RET_32_32(n64,d32,r32,q32) \
			{ \
				__asm mov eax,[DWORD PTR n64] \
				__asm mov edx,[DWORD PTR n64+4] \
				__asm div d32 \
				__asm mov q32,eax \
				__asm mov r32,edx \
			}
	#else
		#define MUL_32_RET_64(a32,b32,r64) \
			asm( \
				"mull %2" \
				: "=a"(((uint64u*)&r64)->lo32), "=d"(((uint64u*)&r64)->hi32) \
				: "m"(b32), "a"(a32) );

		#define DIV_MOD_64_RET_32_32(n64,d32,r32,q32) \
			asm( \
				"divl %2" \
				: "=a"(q32), "=d"(r32) \
				: "rm"(d32), "a"(((uint64u*)&n64)->lo32), "d"(((uint64u*)&n64)->hi32) );
	#endif
#else
	#define MUL_32_RET_64(a32,b32,r64) \
		r64 = (uint64_t) a32 * b32;

	#define DIV_MOD_64_RET_32_32(n64,d32,r32,q32) \
		r32 = n64 % d32; \
		q32 = n64 / d32;
#endif

union uint64u
{
	uint64_t u64;
	struct
	{
#ifdef ARC_LITTLE_ENDIAN
		uint32_t lo32, hi32;
#else
		uint32_t hi32, lo32;
#endif
	};
};

#endif
